
import { IPromise } from 'angular';
import { ISystemorphEntity } from '@systemorph/web';
import { Dictionary } from 'lodash';

export interface IAccessReviewService {
    completeReview(reviewId: string, comment: string): IPromise<any>
    notifyReviewCreated(reviewId: string): IPromise<any>
}

export interface IAccessControlService {
    getSingleAccessObject(accessObjectId: string): IPromise<ISystemorphEntity>
    getGroupMetaInfo(groupId: string, assignmentType: string): IPromise<IGroupMetaInfo>;
    getAccessObjectToBeAddedInGroup(groupSystemName: string, assignmentType: string): IPromise<ISystemorphEntity[]>;
    getGroupsToBeRequested(accessObjectSystemName: string, assignmentType: string): IPromise<ISystemorphEntity[]>;
    getAllGroupAssignmentActivitiesDictionaryCached(): IPromise<Dictionary<IGroupAssignmentActivityModel>>;
    getAllGroupScopeTypeInfosDictionaryCached(): IPromise<Dictionary<IGroupScopeTypeInfoModel>>;
}

export interface IGroupAssignmentService {
    getGroupAssignmentsOfGroup(filter: IGroupAssignmentOfGroupFilter): IPromise<IGroupAssignmentOfGroup[]>;
    getGroupAssignmentLogs(groupId: string, memberId: string, filter: IGroupAssignmentLogsBackendFilter): IPromise<IGroupAssignmentLogModelWithTotalCount>;
    getGroupAssignmentsToBeApprovedByCurrentUser(): IPromise<IGroupAssignmentModel[]>;
    getCurrentUserRequestedGroupAssignments(): IPromise<IGroupAssignmentModel[]>;
    isTooMuchScopesForRequest(scopeIds: string[]): boolean;
    uiSideFilterGroupAssignments(groupAssignments: IGroupAssignmentModel[], filter: IRequestedGroupAssignmentsUiFilter): IGroupAssignmentModel[];
    createGroupAssignments(groupSystemNames: string[], assignmentType: string, memberSystemNames: string[], scopeIds?: string[]): IPromise<ICreateGroupAssignmentSummaryResult>;
    deleteGroupAssignments(groupAssignmentIds: string[]): IPromise<any>;
    approveGroupAssignments(groupAssignmentIds: string[]): IPromise<any>;
    rejectGroupAssignments(groupAssignmentIds: string[]): IPromise<any>;
}

export interface ICreateGroupAssignmentSummaryResult {
    numberOfSucceeded: number;
    duplicateMessages: string[];
    errorMessages: string[];
}

export interface IAccessControlDialogService {
    addOrRequestGroupAssignmentIntoGroup(groupMetaInfo: IGroupMetaInfo): IPromise<any>;
    requestGroupAssignmentForAccessObject(assignmentType: string, accessObjectSystemName: string, accessObjectDisplayName: string): IPromise<any>;
    completeReview(reviewId: string, initialComment: string): IPromise<any>;
}

export interface IAddGroupAssignmentsIntoGroupDialogResult {
    accessObjectSystemNames: string[];
    scopeIds: string[] | null;
}

export interface IRequestGroupAssignmentsForAccessObjectDialogResult {
    groupSystemNames: string[];
    scopeIds: string[] | null;
}

export interface IGroupAssignmentActivityModel {
    value: string;
    displayName: string;
}

export interface IGroupScopeTypeInfoModel {
    name: string;
    displayName: string;
    description: string;
    iconHtml: string;
    membersDirective: string;
    ownersDirective: string;
    memberScopeSelectorDirective: string;
    ownerScopeSelectorDirective: string;
}

export interface IGroupMetaInfo {
    id: string;
    displayName: string;
    systemName: string;
    scopeType: string;
    assignmentType: string;
    isRequestedGroup: boolean;
    canAddAssignments: boolean;
}

export interface IGroupAssignmentOfGroupFilter {
    groupId: string; 
    assignmentType: string;
    assignmentStatus?: string;
    scopeIds: string[] | null;
    memberText: string;
    mode: string;
}

export interface IGroupAssignmentsFilterBase {
    membersSubstring: string;
    groupsSubstring: string;
    assignmentType?: string;
    from?: Date;
    to?: Date;
}

export interface IRequestedGroupAssignmentsUiFilter extends IGroupAssignmentsFilterBase {
    issuersSubstring: string;
    scopesSubstring: string;
    isScopeNull: boolean | null;
}

export interface IGroupAssignmentLogsBackendFilter extends IGroupAssignmentsFilterBase {
    issuersSubstring: string;
    activity: string;
    page: number;
    pageSize: number;
}

export interface IGroupAssignmentLogModelWithTotalCount {
    totalCount: number;
    logs: IGroupAssignmentModel[];
}

export interface IGroupAssignmentOfGroup {
    id: string;
    member: IAccessObjectUiFriendlyModel;
    scopeId: string;
    assignmentStatus: string;
    canDelete: boolean;
}

export interface IGroupAssignmentModel {
    id: string;
    member: IAccessObjectUiFriendlyModel;
    group: IAccessObjectUiFriendlyModel;
    groupScopeType: string;
    scope: IAccessObjectUiFriendlyModel;
    assignmentType: string;
    timestamp: Date | null;
    issuer: IAccessObjectUiFriendlyModel;
    activity: string;
}

export interface IAccessObjectUiFriendlyModel {
    id: string;
    displayName: string;
    systemName: string;
    typeName: string;
}

export interface IGroupUiFriendlyModel extends IAccessObjectUiFriendlyModel {
    scopeType: string;
}