'use strict';

import { 
    register, 
    ILayoutService, 
    IEntityGridArgs, 
    IEntityGridService, 
    IPersisterProviderService, 
    DETAILS_MODE,
    DISPLAY_CONTEXT,
    ITypeLayout,
    ISystemorphEntity,
    IQueryable
} from '@systemorph/web';
import {AC_TYPE_NAME, AC_QUERY_CONFIGURATION} from '../../accessControl.constants'; 
import {AC_COMMON_EVENT} from '../../accessControl.events'; 

register.directive('accessObjectGroups', () => {
    return {
        replace: true,
        restrict: 'E',
        scope: true,
        template: require('./accessObjectGroups.html'),
        controller: AccessObjectGroupsController,
        controllerAs: 'aogCtrl',
        bindToController: {
            accessObjectId: "=",
            assignmentType: "=",
            displayContext: "="
        }
    }
});

class AccessObjectGroupsController {
    // binded
    accessObjectId: string;
    assignmentType: string;
    displayContext: string;

    blockServiceName: string;

    constructor(private $scope: ng.IScope, 
                private layoutService: ILayoutService,
                private entityGridService: IEntityGridService,
                private persisterProviderService: IPersisterProviderService) {
        
        this.blockServiceName = `accessObjectGroupBlock_${this.accessObjectId}_${this.assignmentType}`;

        const displayContext = this.displayContext || DISPLAY_CONTEXT.details;
        this.layoutService.layout(AC_TYPE_NAME.group, displayContext).$promise.then((layout: ITypeLayout) => {
            const entityGridArgs: IEntityGridArgs = {
                scope: this.$scope,
                disablePaging: true,
                entityTypeName: AC_TYPE_NAME.group,
                displayContext: displayContext,
                blockServiceName: this.blockServiceName,
                mode: DETAILS_MODE.display,
                refresh: () => {
                    return this.persisterProviderService.query(layout.collectionName).then((query: IQueryable<ISystemorphEntity>) => {
                        return query.withQueryContext(AC_QUERY_CONFIGURATION.groupsOfAccessObject, { memberId: this.accessObjectId, assignmentType: this.assignmentType});
                    });
                }
            };

            this.entityGridService.createGridOptionsAndPutToScope(entityGridArgs).then(result => {
                this.$scope.$on(AC_COMMON_EVENT.refreshGroupAssignment, () => {
                    result.softUpdate();
                })
            });
        });
    }
}