'use strict';

import { register, IMessageService, DATE_TIME_KIND, DateTimeRangeModel } from '@systemorph/web';
import { IGroupAssignmentModel, IGroupAssignmentService, IRequestedGroupAssignmentsUiFilter } from '../../accessControl.api';
import { AC_COMMON_EVENT } from '../../accessControl.events';
import { IGroupScopeFilterModel } from "../filters";
import { IScope, blockUI, IFormController } from 'angular';

register.directive('currentUserAccessRequests', () => {
    return {
        replace: true,
        scope: true,
        restrict: 'E',
        template: require('./currentUserAccessRequests.html'),
        controller: CurrentUserAccessRequestsController,
        controllerAs: 'cuarCtrl',
        bindToController: {
            hideFilter: "="
        }
    }
});

interface IFilterModel {
    scope: IGroupScopeFilterModel;
    timeRange: DateTimeRangeModel;
    groupsSubstring: string;
    assignmentType?: string;
}

class CurrentUserAccessRequestsController {
    //binded
    hideFilter: boolean;

    blockName: string;
    filterModel: IFilterModel;
    filteredItems: IGroupAssignmentModel[];

    filterForm: IFormController;

    readonly dateTimeKind: string = DATE_TIME_KIND.utc;
    readonly dateTimeFormat: string = 'short';

    private allItems: IGroupAssignmentModel[];
    
    constructor(private $scope: IScope,
                private blockUI: blockUI.IBlockUIService,
                private messageService: IMessageService,
                private groupAssignmentService: IGroupAssignmentService) {
        this.blockName = `currentUserAccessRequests_blockName`;
        this.filterModel = {
            groupsSubstring: '',
            assignmentType: null,
            scope: {
                substring: '',
                isNull: null
            },
            timeRange: {

            }
        }
        this.setItemsAsync();

        this.$scope.$on(AC_COMMON_EVENT.refreshGroupAssignment, () => {
            this.setItemsAsync();
        })
    }

    private setItemsAsync() {
        this.blockUI.instances.get(this.blockName).start("Loading assignments...")

        const promise = this.groupAssignmentService.getCurrentUserRequestedGroupAssignments();

        return promise
            .then(groupAssignments => {
                this.allItems = groupAssignments;
            })
            .catch(() => {
                this.messageService.alertError(`Error on loading group assignments`);
                this.allItems = [];
            })
            .finally(() => {
                this.blockUI.instances.get(this.blockName).stop();
                this.filterChanged();
            })
    }

    filterChanged() {
        if (this.filterForm.$valid) {
            const uiFilterModel: IRequestedGroupAssignmentsUiFilter = {
                assignmentType: this.filterModel.assignmentType,
                groupsSubstring: this.filterModel.groupsSubstring,
                issuersSubstring: null,
                membersSubstring: null,
                from: this.filterModel.timeRange.from,
                to: this.filterModel.timeRange.to,
                isScopeNull: this.filterModel.scope.isNull,
                scopesSubstring: this.filterModel.scope.substring
            }

            this.filteredItems = this.groupAssignmentService.uiSideFilterGroupAssignments(this.allItems, uiFilterModel)
                        .map(x => ({...x}));//TRICK to avoid ng-repeat issue
        }
    }
}