
import { IEditMultiSelectOptions, IDimensionNameFilter, dimensionBestIndexMatchFilter, ISystemorphEntity } from '@systemorph/web';
import { IGroupScopeTypeInfoModel, IGroupMetaInfo, IAddGroupAssignmentsIntoGroupDialogResult } from "../../accessControl.api";
import { getScopeSelectorDirectiveFromAssignmentType } from "../../utils/helpers";
import { AC_COMMON_EVENT } from "../../accessControl.events";
import { IAccessObjectIconHtmlByOdataInstanceFilter } from "../../utils/accessObjectIconHtmlByOdataInstanceFilter";
import { IScope, IFilterService, IPromise } from "angular";
import { Dictionary } from "lodash";
import { IModalServiceInstance } from 'angular-ui-bootstrap';

export class AddToGroupDialogController {
    options: IEditMultiSelectOptions<ISystemorphEntity>;
    scopeIds: string[] | null;
    selectedAccessObjects: ISystemorphEntity[];
    isDirtyAccessObjects: boolean;
    scopeSelectorDirectiveName: string;

    /*@ngInject*/
    constructor(private $scope: IScope,
                private $filter: IFilterService, 
                private $uibModalInstance: IModalServiceInstance, 
                private groupMetaInfo: IGroupMetaInfo, 
                private groupScopeTypeInfosDictionary: Dictionary<IGroupScopeTypeInfoModel>, 
                private accessObjects: ISystemorphEntity[]) {

        this.scopeSelectorDirectiveName = getScopeSelectorDirectiveFromAssignmentType(this.groupScopeTypeInfosDictionary[this.groupMetaInfo.scopeType], this.groupMetaInfo.assignmentType)
        this.scopeIds = this.scopeSelectorDirectiveName ? [] : null;
        this.options = {
            allValues: this.accessObjects,
            getItemId: item => item.SystemName,
            getItemDisplayName: item => `<span class="sm-access-control-object__icon">
                                            ${this.$filter<IAccessObjectIconHtmlByOdataInstanceFilter>("accessObjectIconHtmlByOdataInstance")(item, this.groupScopeTypeInfosDictionary)}
                                         </span> ${this.$filter<IDimensionNameFilter>("dimensionName")(item)}`,
            bestIndexMatchFilter: dimensionBestIndexMatchFilter
        }
        this.selectedAccessObjects = []
        this.isDirtyAccessObjects = false;

        // it is api for the confirmationService
        this.$scope.ok = () => {
            this.isDirtyAccessObjects = true;
            this.$scope.$broadcast(AC_COMMON_EVENT.dirtifyScopeSelector);
            if (this.selectedAccessObjects.length && (!this.scopeSelectorDirectiveName || this.scopeIds.length)) {
                const ret: IAddGroupAssignmentsIntoGroupDialogResult = {
                    accessObjectSystemNames: this.selectedAccessObjects.map(x => x.SystemName),
                    scopeIds: this.scopeIds
                }
                this.$uibModalInstance.close(ret);
            }
        };

        // it is api for the confirmationService
        this.$scope.cancel = () => {
            this.$uibModalInstance.dismiss();
        };
    }

    onAccessObjectsChanged() {
        this.isDirtyAccessObjects = true;
    }
}