
import { IEditMultiSelectOptions, IDimensionNameFilter, dimensionBestIndexMatchFilter, ISystemorphEntity } from '@systemorph/web';
import { IGroupScopeTypeInfoModel, IRequestGroupAssignmentsForAccessObjectDialogResult } from "../../accessControl.api";
import { AC_COMMON_EVENT } from "../../accessControl.events";
import { IAccessObjectIconHtmlByOdataInstanceFilter, getScopeSelectorDirectiveFromAssignmentType } from "../../utils";
import { IScope, IFilterService, IPromise } from "angular";
import { Dictionary } from "lodash";
import { IModalServiceInstance } from 'angular-ui-bootstrap';

interface IGroupScopeTypeInfoModelExtended extends IGroupScopeTypeInfoModel {
    scopeSelectorDirectiveName: string;
}

export class RequestForAccessObjectDialogController {
    activeScopeTypeInfo: IGroupScopeTypeInfoModelExtended;
    allScopeTypeInfos: IGroupScopeTypeInfoModelExtended[];

    options: IEditMultiSelectOptions<ISystemorphEntity>;
    scopeIds: string[] | null;
    selectedGroups: ISystemorphEntity[];
    isDirtyGroups: boolean;
    isEmpty: boolean;

    /*@ngInject*/
    constructor(private $scope: IScope,
                private $filter: IFilterService,
                private $uibModalInstance: IModalServiceInstance,
                private assignmentType: string,
                private groupScopeTypeInfosDictionary: Dictionary<IGroupScopeTypeInfoModel>,
                private groups: ISystemorphEntity[]) {

        this.isDirtyGroups = false;
        this.isEmpty = false;
        this.allScopeTypeInfos = Object.keys(this.groupScopeTypeInfosDictionary)
            .map(key => this.groupScopeTypeInfosDictionary[key])
            .map(x => (<IGroupScopeTypeInfoModelExtended>{
                ...x,
                scopeSelectorDirectiveName: getScopeSelectorDirectiveFromAssignmentType(x, this.assignmentType)
            }));

        this.onChangeScopeType(this.allScopeTypeInfos[0], undefined)

        // it is api for the confirmationService
        this.$scope.ok = () => {
            this.isDirtyGroups = true;
            this.$scope.$broadcast(AC_COMMON_EVENT.dirtifyScopeSelector);
            if (this.selectedGroups.length && (!this.activeScopeTypeInfo.scopeSelectorDirectiveName || this.scopeIds.length)) {
                const ret: IRequestGroupAssignmentsForAccessObjectDialogResult = {
                    groupSystemNames: this.selectedGroups.map(x => x.SystemName),
                    scopeIds: this.scopeIds
                }
                this.$uibModalInstance.close(ret);
            }
        };

        // it is api for the confirmationService
        this.$scope.cancel = () => {
            this.$uibModalInstance.dismiss();
        };
    }

    onGroupsChanged() {
        this.isDirtyGroups = true;
    }

    onChangeScopeType(newVal: IGroupScopeTypeInfoModelExtended, oldVal: IGroupScopeTypeInfoModelExtended) {
        if (newVal !== oldVal) {
            this.activeScopeTypeInfo = newVal;
            this.scopeIds = newVal.scopeSelectorDirectiveName ? [] : null;
            this.isDirtyGroups = false;

            const availableGroups = this.groups.filter(x => x["ScopeType"] === newVal.name);
            this.isEmpty = availableGroups.length === 0;

            this.options = {
                allValues: availableGroups,
                getItemId: item => item.SystemName,
                getItemDisplayName: item => `<span class="sm-access-control-object__icon">
                                            ${this.$filter<IAccessObjectIconHtmlByOdataInstanceFilter>("accessObjectIconHtmlByOdataInstance")(item, this.groupScopeTypeInfosDictionary)}
                                         </span> ${this.$filter<IDimensionNameFilter>("dimensionName")(item)}`,
                bestIndexMatchFilter: dimensionBestIndexMatchFilter
            }
            this.selectedGroups = []
        };
    }
}