'use strict';

import { register } from '@systemorph/web';
import { IAccessControlService } from "../../accessControl.api";
import { ITimeoutService } from 'angular';

register.directive("groupAssignmentLogActivityFilter", () => {
    return {
        restrict: 'E',
        replace: true,
        require: "ngModel",
        scope: true,
        controllerAs: 'galaCtrl',
        bindToController: {
            activity: "=ngModel",
            onChange: "&"
        },
        template: require('./groupAssignmentLogActivityFilter.html'),
        controller: GroupAssignmentnLogActivityFilterController
    };
});

interface IItem {
    value: string;
    displayName: string; 
}

class GroupAssignmentnLogActivityFilterController {
    //binded
    activity: string;
    onChange: () => void;

    selected: IItem;
    allActivitites: IItem[];

    constructor(private $timeout: ITimeoutService, private accessControlService: IAccessControlService) {
        this.accessControlService.getAllGroupAssignmentActivitiesDictionaryCached().then(dictionary => {
            this.allActivitites = [
                {
                    value: null,
                    displayName: "All"
                },
                ...Object.keys(dictionary).map(v => <IItem>({ value: dictionary[v].value, displayName: dictionary[v].displayName}))
            ];
            this.selected = this.allActivitites.find(x => x.value === this.activity) || this.allActivitites[0];
        })
    }

    changeHandler(newVal: IItem, oldVal: IItem) {
        if (newVal !== oldVal) {
            this.activity = newVal.value;
            // $timeout is required due to ngModel value propogation is performed in several digest cycles. 
            // ie parent scope will always get values from previous digest cycle without $timeout.
            this.$timeout(() => {
                if (this.onChange)
                    this.onChange();

            });
        }
    }
}