'use strict';

import { register } from '@systemorph/web';
import { ICreateGroupAssignmentSummaryResult } from "../../accessControl.api";
import { IScope } from 'angular';
import { isEmpty } from 'lodash';

register.directive('createAssignmentsSummary', () => {
    return {
        replace: true,
        restrict: 'E',
        scope: true,
        template: require('./createAssignmentsSummary.html'),
        controller: CreateAssignmentsSummaryController,
        controllerAs: 'ctrl',
        bindToController: {
            summary: '='
        }
    }
});

class CreateAssignmentsSummaryController {
    // binded
    summary: ICreateGroupAssignmentSummaryResult;

    successText: string;
    duplicateMessages: string[];
    errorMessages: string[];

    constructor(private $scope: IScope) {
        if (this.summary.numberOfSucceeded) {
            this.successText = this.summary.numberOfSucceeded > 1
                ? `${this.summary.numberOfSucceeded} requests submitted.`
                : `${this.summary.numberOfSucceeded} request submitted.`
        }

        if (!isEmpty(this.summary.duplicateMessages)) {
            this.duplicateMessages = this.summary.duplicateMessages.map(x => this.replaceNewLine(x));
        }

        if (!isEmpty(this.summary.errorMessages)) {
            this.errorMessages = this.summary.errorMessages.map(x => this.replaceNewLine(x));
        }
    }

    private replaceNewLine(text: string) {
        return text.replace(/\n/g, `<br/>`)
    }
}