'use strict';

import { 
    register, 
    ILayoutService, 
    IEntityGridArgs, 
    IEntityGridService, 
    IPersisterProviderService, 
    DETAILS_MODE,
    ITypeLayout,
    ISystemorphEntity,
    IQueryable,
    COMMON_EVENT
} from '@systemorph/web';
import {AC_TYPE_NAME, AC_DISPLAY_CONTEXT, AC_REVIEW_STATUS } from '../../accessControl.constants'; 
import { IStateService, IStateParamsService } from 'angular-ui-router';

register.directive('accessReview', () => {
    return {
        replace: true,
        restrict: 'E',
        scope: true,
        template: require('./accessReview.html'),
        controller: AccessReviewController,
        controllerAs: 'arCtrl',
        bindToController: {
            groupId: "=",
            status: "=",
        }
    }
});

class AccessReviewController {
    // binded
    groupId: string;
    status: string;

    blockServiceName: string;

    constructor(private $scope: ng.IScope, 
                private layoutService: ILayoutService,
                private $state: IStateService,
                private $stateParams: IStateParamsService,
                private entityGridService: IEntityGridService,
                private persisterProviderService: IPersisterProviderService) {
        
        this.blockServiceName = `accessReviewBlock_${this.groupId}_${this.status}`;
        const displayContext = this.groupId
                                ? AC_DISPLAY_CONTEXT.accessReviewsOfSingleGroup
                                : AC_DISPLAY_CONTEXT.allAccessReviews;

        this.layoutService.layout(AC_TYPE_NAME.accessReview, displayContext).$promise.then((layout: ITypeLayout) => {
            const entityGridArgs: IEntityGridArgs = {
                scope: this.$scope,
                disablePaging: false,
                entityTypeName: AC_TYPE_NAME.accessReview,
                displayContext: displayContext,
                blockServiceName: this.blockServiceName,
                mode: DETAILS_MODE.display,
                refresh: () => {
                    return this.persisterProviderService.query(layout.collectionName).then((query: IQueryable<ISystemorphEntity>) => {
                        query = this.status === AC_REVIEW_STATUS.pending
                                    ? query.filter(`it.ReviewCompletionDate == null`)
                                    : query.filter(`it.ReviewCompletionDate != null`);
                        if (this.groupId)
                            query = query.filter(`it.Group.Id == '${this.groupId}'`);
                        return query;
                    });
                },
                stateParams: this.$stateParams,
                stateName: this.$state.current.name,
            };

            this.entityGridService.createGridOptionsAndPutToScope(entityGridArgs).then(result => {
                this.$scope.$on(COMMON_EVENT.commitDataToServer, () => {
                    result.softUpdate();
                })
            });
        });
    }
}