'use strict';

import { setPageTitle, IPersisterProviderService, ILayoutService, ISystemorphEntity } from '@systemorph/web';
import { IAccessControlService, IAccessControlDialogService } from "../../accessControl.api";
import { AC_COMMON_EVENT } from "../../accessControl.events";
import { getAddGroupAssignmentTitleTextForAccessObject } from "../../utils/helpers";
import { IScope, IFilterService, blockUI, IPromise } from 'angular';
import { IStateParamsService } from 'angular-ui-router';

// note this should not be a public api
export abstract class AccessObjectGroupsControllerBase {
    accessObjectId: string;
    accessObjectRequestArguments: { displayName: string, systemName: string };
    requestGroupAssignmentText: string;
    errorMessage: string;

    constructor(
        protected $scope: IScope,
        protected $stateParams: IStateParamsService,
        protected $filter: IFilterService,
        protected accessControlService: IAccessControlService,
        protected accessControlDialogService: IAccessControlDialogService,
        protected persisterProviderService: IPersisterProviderService,
        protected layoutService: ILayoutService,
        protected blockUI: blockUI.IBlockUIService,
        protected assignmentType: string,
        protected displayContext: string,
        protected title: string
    ) {
        this.accessObjectId = this.$stateParams["entityId"];
        this.requestGroupAssignmentText = getAddGroupAssignmentTitleTextForAccessObject(this.assignmentType, undefined);

        this.blockUI.start("Please wait");
        this.loadAccessObject(this.accessObjectId).then(accessObject => {
            this.accessObjectRequestArguments = { displayName: accessObject.DisplayName, systemName: accessObject.SystemName};
            setPageTitle(this.$scope, this.title);
        }).catch(() => {
            this.title = `Access Object was not found`;
            setPageTitle(this.$scope, this.title);
            this.errorMessage = `Cannot find Access Object '${this.accessObjectId}'`
        }).finally(() => {
            this.blockUI.stop();
        })
    }

    requestGroupAssignment() {
        this.accessControlDialogService.requestGroupAssignmentForAccessObject(this.assignmentType, this.accessObjectRequestArguments.systemName, this.accessObjectRequestArguments.displayName)
            .then(ret => {
                this.$scope.$broadcast(AC_COMMON_EVENT.refreshGroupAssignment);
            });
    }

    protected loadAccessObject(accessObjectId: string): IPromise<ISystemorphEntity> {
        return this.accessControlService.getSingleAccessObject(accessObjectId);
    }
}