'use strict';

import { register, setPageTitle, IUserService, STATE } from '@systemorph/web';
import { getAddGroupAssignmentTitleTextForAccessObject } from "../../utils";
import { AC_ASSIGNMENT_TYPE, AC_STATE } from "../../accessControl.constants";
import { AC_COMMON_EVENT } from "../../accessControl.events";
import { IAccessControlDialogService } from "../../accessControl.api";
import { IScope } from 'angular';
import { IStateParamsService, IStateService } from 'angular-ui-router';

class CurrentUserAccessRequestsController {
    title: string;
    requestGroupAssignmentText: string;
    assignmentType: string;

    /*@ngInject*/
    constructor(private $scope: IScope,
                private userService: IUserService,
                private $state: IStateService,
                private $stateParams: IStateParamsService,
                private accessControlDialogService: IAccessControlDialogService) {
        
        this.title = "My pending requests";
        setPageTitle(this.$scope, this.title);

        this.assignmentType = AC_ASSIGNMENT_TYPE.member;
        this.requestGroupAssignmentText = getAddGroupAssignmentTitleTextForAccessObject(this.assignmentType, undefined);

        const openRequestDialogParamName = "openRequestDialog";
        if (this.$stateParams[openRequestDialogParamName] === "true") {
            this.requestGroupAssignment();
            this.$state.go(this.$state.current.name, { [openRequestDialogParamName]: undefined }, { notify: false });
        }
    }

    requestGroupAssignment() {
        this.userService.getCurrentUserInfo().then(currentUserInfo => {
            this.accessControlDialogService.requestGroupAssignmentForAccessObject(this.assignmentType, currentUserInfo.systemName, currentUserInfo.displayName)
                .then(ret => {
                    this.$scope.$broadcast(AC_COMMON_EVENT.refreshGroupAssignment);
                });
        })
    }
}

register.state({
    name: AC_STATE.currentUserAccessRequests,
    parent: STATE.defaultLayout,
    template: require('./currentUserAccessRequests.html'),
    url: '/currentUserAccessRequests?openRequestDialog',
    controllerAs: 'pageCtrl',
    controller: CurrentUserAccessRequestsController,
    suppressReloadMessage: true
})