'use strict';

import { register, IDimensionNameFilter, ISystemorphEntity, STATE } from '@systemorph/web';
import { GroupRelationshipControllerBase} from "./groupRelationshipControllerBase";
import { AC_ASSIGNMENT_TYPE, AC_STATE } from "../../accessControl.constants";
import { IGroupMetaInfo, IAccessControlService, IGroupScopeTypeInfoModel, IAccessControlDialogService } from "../../accessControl.api";
import { IScope, blockUI, IQService, IFilterService} from 'angular';
import { IStateService, IStateParamsService } from 'angular-ui-router';

class GroupOwnersController extends GroupRelationshipControllerBase {
    /*@ngInject*/
    constructor(
        $scope: IScope,
        $state: IStateService,
        $stateParams: IStateParamsService,
        $q: IQService,
        private $filter: IFilterService,
        accessControlService: IAccessControlService,
        accessControlDialogService: IAccessControlDialogService,
        blockUI: blockUI.IBlockUIService
    ) {
        super($scope, $q, $state, $stateParams, accessControlService, accessControlDialogService, blockUI, AC_ASSIGNMENT_TYPE.owner);
    }

    protected getTitleByMetaInfo(group: IGroupMetaInfo): string {
        const dimensionStub: ISystemorphEntity = <any>{
            DisplayName: group.displayName,
            SystemName: group.systemName
        };

        return `${this.$filter<IDimensionNameFilter>("dimensionName")(dimensionStub)} owners`;
    }
    protected getGroupAssignmentsDirectiveName(groupScopeTypeInfo: IGroupScopeTypeInfoModel): string {
        return groupScopeTypeInfo.ownersDirective;
    }
}

register.state({
    name: AC_STATE.groupOwners,
    parent: STATE.defaultLayout,
    url: '/groupOwners/:mode/:entityId?contextUserId&reviewId',
    template: require('./groupRelationship.html'),
    controllerAs: 'grCtrl',
    controller: GroupOwnersController,
    suppressReloadMessage: true
})