'use strict';

import { setPageTitle, ISystemorphEntity, STATE } from '@systemorph/web';
import { IGroupMetaInfo, IAccessControlService, IGroupScopeTypeInfoModel, IAccessControlDialogService } from "../../accessControl.api";
import { AC_COMMON_EVENT } from "../../accessControl.events";
import { getAddGroupAssignmentTitleTextForGroup } from "../../utils";
import { IScope, blockUI, IQService, IPromise } from 'angular';
import { IStateService, IStateParamsService } from 'angular-ui-router';

// note this should not be a public api
export abstract class GroupRelationshipControllerBase {
    title: string;
    errorMessage: string;
    groupMetaInfo: IGroupMetaInfo
    groupAssignmentsDirectiveName: string;
    contextUserName: string;
    addToGroupButtonText: string;
    canAdd: boolean;
    mode: string;
    reviewUrl: string;

    constructor(
        private $scope: IScope,
        private $q: IQService,
        private $state: IStateService,
        private $stateParams: IStateParamsService,
        private accessControlService: IAccessControlService,
        private accessControlDialogService: IAccessControlDialogService,
        private blockUI: blockUI.IBlockUIService,
        assignmentType: string
    ) {
        this.mode = this.$stateParams["mode"];
        const groupId: string = this.$stateParams["entityId"];
        const contextUserId: string = this.$stateParams["contextUserId"];
        const reviewId: string = this.$stateParams["reviewId"];
        if (reviewId) {
            this.reviewUrl = this.$state.href(STATE.details, { entityId: reviewId });
        }

        this.blockUI.start("Please wait");
        this.$q.all([
            this.loadGroupMetaInfo(groupId, assignmentType),
            this.loadUser(contextUserId),
            this.accessControlService.getAllGroupScopeTypeInfosDictionaryCached()
        ]).then(([groupMetaInfo, contextUser, dictionary]) => {
            this.groupMetaInfo = groupMetaInfo;
            this.groupAssignmentsDirectiveName = this.getGroupAssignmentsDirectiveName(dictionary[this.groupMetaInfo.scopeType]);
            this.contextUserName = contextUser ? contextUser.SystemName : null;
            this.addToGroupButtonText = getAddGroupAssignmentTitleTextForGroup(assignmentType, this.groupMetaInfo.isRequestedGroup, null /* skip group name */);
            this.canAdd = this.groupMetaInfo.canAddAssignments;
            this.title = this.getTitleByMetaInfo(this.groupMetaInfo);
            setPageTitle(this.$scope, this.title);
        }).catch(() => {
            this.title = this.getNotFoundTitle();
            setPageTitle(this.$scope, this.title);
            this.errorMessage = `Cannot find group '${groupId}'`
        }).finally(() => {
            this.blockUI.stop();
        })
    }

    protected onAddAssignment() {
        this.accessControlDialogService.addOrRequestGroupAssignmentIntoGroup(this.groupMetaInfo)
            .then(ret => {
                this.$scope.$broadcast(AC_COMMON_EVENT.refreshGroupAssignment);
            });
    }

    protected abstract getTitleByMetaInfo(groupMetaInfo: IGroupMetaInfo): string;
    protected abstract getGroupAssignmentsDirectiveName(groupScopeTypeInfo: IGroupScopeTypeInfoModel): string;

    protected getNotFoundTitle(): string {
        return `Group was not found`;
    }

    protected loadGroupMetaInfo(groupId: string, assignmentType: string): IPromise<IGroupMetaInfo> {
        return this.accessControlService.getGroupMetaInfo(groupId, assignmentType);
    }

    protected loadUser(userId: string): IPromise<ISystemorphEntity> {
        if (!userId)
            return this.$q.when(null);

        return this.accessControlService.getSingleAccessObject(userId);
    }
}
