'use strict';

import {
    register,
    setPageTitle,
    ILayoutService,
    IPersisterProviderService,
    IEntityService,
    DISPLAY_CONTEXT,
    objectFromQueryString,
    objectToQueryString,
    STATE
} from '@systemorph/web';
import { IGroupAssignmentLogsBackendFilter } from '../../accessControl.api';
import { AC_STATE } from '../../accessControl.constants';
import { IPromise, IScope, ITimeoutService, IQService } from 'angular';
import { IStateParamsService, IStateService } from 'angular-ui-router';

export class GroupAssignmentLogsController {
    title: string;
    stateParamsFilter: IGroupAssignmentLogsBackendFilter;
    groupId: string;
    memberId: string;

    private readonly filterQueryParamKey = 'filter';

    /*@ngInject*/
    constructor(protected $scope: IScope,
                protected $timeout: ITimeoutService,
                protected $q: IQService,
                protected $state: IStateService,
                protected $stateParams: IStateParamsService,
                protected layoutService: ILayoutService,
                protected persisterProviderService: IPersisterProviderService,
                protected entityService: IEntityService) {

        this.setTitleAndExtraParams();
        this.stateParamsFilter = objectFromQueryString(this.$stateParams[this.filterQueryParamKey]) || {};
    }

    protected setTitleAndExtraParams() {
        this.title = "Assignment logs";
        setPageTitle(this.$scope, this.title);

        this.groupId = this.$stateParams['groupId'];
        this.memberId = this.$stateParams['memberId'];
        this.$q.all([
            this.groupId ? this.loadEntityName(this.groupId) : this.$q.when(null),
            this.memberId ? this.loadEntityName(this.memberId) : this.$q.when(null)
        ]).then(([groupName, memberName]) => {
            if (groupName) {
                this.title = `${this.title} of ${groupName}`;
            }
            if (memberName) {
                this.title = `${this.title} for ${memberName}`;
            }
            setPageTitle(this.$scope, this.title);
        })
    }

    filterChanged(filter: IGroupAssignmentLogsBackendFilter) {
        const value: string = objectToQueryString(filter);
        this.$state.go(this.$state.current.name, { [this.filterQueryParamKey]: value }, { notify: false });
    }

    protected loadEntityName(entityId: string): IPromise<string> {
        return this.entityService.entityInfo(entityId).then(entityInfo => {
            return this.layoutService.layout(entityInfo.typeName, DISPLAY_CONTEXT.details).$promise.then(layout => {
                return this.persisterProviderService.query(layout.collectionName).then(query => {
                    return query.find(entityId, true).then(entity => {
                        return entity.DisplayName;
                    })
                })
            })
        });
    }
}

register.state({
    name: AC_STATE.groupAssignmentLogs,
    parent: STATE.defaultLayout,
    url: "/groupAssignmentLogs?filter&memberId&groupId",
    template: require('./groupAssignmentLogs.html'),
    controllerAs: 'pageCtrl',
    controller: GroupAssignmentLogsController,
    suppressReloadMessage: true
})