'use strict';

import { register, setPageTitle, IDimensionNameFilter, STATE } from '@systemorph/web';
import { IAccessControlService } from "../../accessControl.api";
import { getAccessReviewsTitleText } from "../../utils/helpers";
import { IScope, blockUI, IFilterService } from 'angular';
import { IStateParamsService } from 'angular-ui-router';
import { AC_STATE } from '../../accessControl.constants';

class AccessReviewController {
    title: string;
    groupId: string;
    status: string;
    hasError: boolean;
    errorMessage: string;

    /*@ngInject*/
    constructor(
        private $scope: IScope,
        private $stateParams: IStateParamsService,
        private $filter: IFilterService,
        private accessControlService: IAccessControlService,
        private blockUI: blockUI.IBlockUIService
    ) {
        this.groupId = this.$stateParams["groupId"];
        this.status = this.$stateParams["status"];

        this.setTitleAndPageTitle(getAccessReviewsTitleText(this.status))
        this.hasError = false;

        if (this.groupId) {
            this.blockUI.start("Please wait");
            this.accessControlService.getSingleAccessObject(this.groupId).then(group => {
                const groupName = this.$filter<IDimensionNameFilter>('dimensionName')(group);
                this.setTitleAndPageTitle(getAccessReviewsTitleText(this.status, groupName))
            }).catch(() => {
                this.setTitleAndPageTitle(`Group was not found`)
                this.hasError = true;
                this.errorMessage = `Cannot find Group '${this.groupId}'`
            }).finally(() => {
                this.blockUI.stop();
            })
        }
    }

    private setTitleAndPageTitle(text: string) {
        this.title = text;
        setPageTitle(this.$scope, text);
    } 
}

register.state({
    name: AC_STATE.accessReview,
    parent: STATE.defaultLayout,
    url: '/accessReview/:status?groupId',
    template: require('./accessReview.html'),
    controllerAs: 'pageCtrl',
    controller: AccessReviewController,
    suppressReloadMessage: true
})