
import { AC_TYPE_NAME, AC_ASSIGNMENT_TYPE, AC_REVIEW_STATUS } from "../accessControl.constants";
import { IGroupScopeTypeInfoModel } from "../accessControl.api";
import { includes, Dictionary } from 'lodash';

export const getAccessObjectIconHtml = (type: string, scopeType?: string, groupScopeTypeInfosDictionary?: Dictionary<IGroupScopeTypeInfoModel>): string => {
    switch (type) {
        case AC_TYPE_NAME.group:
                return groupScopeTypeInfosDictionary[scopeType].iconHtml;
        case AC_TYPE_NAME.user:
            return `<i class="material-icons">person</i>`;
        default:
            return `<i class="material-icons">help</i>`;
    }
}

export const getScopeSelectorDirectiveFromAssignmentType = (scopeTypeInfo: IGroupScopeTypeInfoModel, assignmentType: string) => {
    switch (assignmentType) {
        case AC_ASSIGNMENT_TYPE.owner:
            return scopeTypeInfo.ownerScopeSelectorDirective;
        case AC_ASSIGNMENT_TYPE.member:
            return scopeTypeInfo.memberScopeSelectorDirective;
        default:
            return scopeTypeInfo.memberScopeSelectorDirective;
    }
}

export const getGroupAssignmentLogActivityClass = (activity: string): string => {
    const activityLower = !!activity ? activity.toLowerCase() : "";
    if (includes(activityLower, "reject") || includes(activityLower, "remove")) {
        return "text-danger";
    }
    if (includes(activityLower, "requested") || includes(activityLower, "added")) {
        return "text-success";
    }
    return "text-default";
}

export const getAddGroupAssignmentTitleTextForGroup = (assignmentType: string, isRequiested: boolean, groupName?: string): string => {
    const baseText = assignmentType === AC_ASSIGNMENT_TYPE.member
        ? (isRequiested ? "Request membership" : "Add member")   
        : (isRequiested ? "Request ownership" : "Add owner")

    if (groupName) {
        return `${baseText} to ${groupName} group`;
    }

    return baseText;
}

export const getAddGroupAssignmentTitleTextForAccessObject = (assignmentType: string, accessObjectName?: string): string => {
    const baseText = assignmentType === AC_ASSIGNMENT_TYPE.member
        ? "Request membership"
        : "Request ownership";

    if (accessObjectName) {
        return `${baseText} for ${accessObjectName}`
    }

    return baseText;
}

export const getAccessReviewsTitleText = (status: string, groupDisplayName?: string): string => {
    const statusText = status === AC_REVIEW_STATUS.pending
        ? "Pending"
        : "Completed";

    const baseText = `${statusText} reviews`;
    if (groupDisplayName) {
        return `${baseText} for ${groupDisplayName}`
    }
    return baseText;
}

