'use strict';

import { register, IMessageService, ITypeLayout } from "@systemorph/web";
import { ILogOnModel, IChangePasswordModel, ILogOnResponse, IAccountResponse, IAccountService, IAuthenticationData } from './account.api';
import { ICacheObject, ILocationService, IWindowService, IHttpService, IPromise } from 'angular';

class AccountService implements IAccountService {
    cache: ICacheObject;
    constructor(
        private $http: IHttpService,
        private $window: IWindowService,
        private messageService: IMessageService,
        private $location: ILocationService) {
    }

    getAuthenticationData(): IPromise<IAuthenticationData> {
        return this.$http.get<IAuthenticationData>('/api/account/authenticationData', {}).then(result => result.data);
    }

    logOnLayout() {
        return this.layout('/api/account/logOnLayout');
    }

    logonUser(model: ILogOnModel) {
        return this.$http.post<ILogOnResponse>('/api/account/logon', model).then((response) => {
            if (response.data.success) {
                this.$window.location.href = model.ReturnUrl ? model.ReturnUrl : "/";
                return response.data;
            } else {
                this.messageService.alertError(response.data.error);
                throw response.data;
            }
        });
    }

    changePasswordLayout() {
        return this.layout('/api/account/changePasswordLayout');
    }

    changePassword(model: IChangePasswordModel, returnUrl: string) {
        return this.$http.post<IAccountResponse>('/api/account/changePassword', model).then((response) => {
            if (response.data.success) {
                this.$location.path(returnUrl ? returnUrl : "/").replace();
                return response.data;
            } else {
                this.messageService.alertError(response.data.error);
                throw response.data;
            }
        });
    }

    initializePassword = (entityId: string) => {
        return this.$http.post(`/api/account/initializePassword/${ entityId }`, null)
            .then(({ data }) => {
                return data;
            })
            .catch<string>(({ data: error }): string => {
                throw error;
            });
    };

    private layout(url: string) {
        return this.$http.get<ITypeLayout>(url, {}).then((layout) => {
            return layout.data;
        }).catch<ITypeLayout>(() => {
            throw new Error("No layout was found");
        });
    }
}

register.service('accountService', AccountService); 