

import { register, IWorkFlowService, IMessageService, IAjaxMenuItemScope } from '@systemorph/web';

register.directive('oktaLogoutMenuItem', () => {
    return {
        replace: true,
        restrict: 'E',
        template: require('@systemorph/web/app/menus/items/ajaxMenuItem.html'),
        transclude: true,
        controller: ($scope: IAjaxMenuItemScope, $http: ng.IHttpService, messageService: IMessageService, workFlowService: IWorkFlowService) => {

            $scope.executeAjaxCall = () => {
                if (!$scope.item.disabled) {
                    $http.post<string>($scope.item.url, ($scope.item.parameters || {}), { })
                        .then(result => {
                            var url = result.data;
                            if (url) {
                                $http.delete<any>(`${ url }/api/v1/sessions/me`, { withCredentials: true })
                                    .then(() => {
                                        window.location.reload();
                                    })
                                    .catch(reason => {
                                        var title, message;
                                        switch (reason.status) {
                                            case 403:
                                                title = reason.data.message;
                                                message = 'Please login to your Okta organization';
                                                break;
                                            default:
                                                title = 'Invalid URL or Cross-Origin Request Blocked';
                                                message = 'You must explicitly add this site (' + window.location.origin + ') to the list of allowed websites in your administrator UI';
                                                break;
                                        }
                                        messageService.alertError(`${ title }: ${ message }`);
                                    })
                            }
                        }).catch((result: { data: string }) => {
                            if (result.data) {
                                messageService.alertError(result.data);
                            }
                        });
                }
            };
        }
    };
});