'use strict';

import { register, setPageTitle, ITypeLayout, COMMON_EVENT, DETAILS_MODE, DISPLAY_CONTEXT, STATE } from "@systemorph/web";
import { IChangePasswordModel, IAccountService } from '../account.api';
import { ACCOUNT_STATE } from '../account.constants';
import { IStateParamsService } from 'angular-ui-router';
import { IScope, IFormController } from 'angular';

class Controller {
    model: IChangePasswordModel;
    mode: string;
    displayContext: string;

    /*@ngInject*/
    constructor(private $scope: IScope,
                private accountService: IAccountService,
                public changePasswordLayout: ITypeLayout,
                private $stateParams: IStateParamsService) {
    
        setPageTitle($scope, "Change password");
    
        this.mode = DETAILS_MODE.edit;
        this.displayContext = DISPLAY_CONTEXT.details;
        this.model = {};
    }

    submit() {
        const changePasswordForm: IFormController = this.$scope.changePasswordForm; // form is not controller property due to wired exception
        if (!changePasswordForm)
            throw "Change password form was not found";

        this.$scope.$broadcast(COMMON_EVENT.dirtifyPropertySuite);

        if (changePasswordForm.$invalid)
            return;

        this.accountService.changePassword(this.model, this.$stateParams.ReturnUrl);
    };
}

register.state({
    name: ACCOUNT_STATE.changePassword,
    parent: STATE.defaultLayout,
    url: '/changePassword?ReturnUrl',
    template: require('./changePassword.html'),
    suppressReloadMessage: true,
    resolve: {
        changePasswordLayout: /*@ngInject*/(accountService: IAccountService) => {
            return accountService.changePasswordLayout()
        }
    },
    controllerAs: "ctrl",
    controller: Controller
});
