'use strict';

import { register, setPageTitle, ITypeLayout, COMMON_EVENT, DETAILS_MODE, DISPLAY_CONTEXT } from '@systemorph/web';
import { AC_APPLICATION_STAGE } from '../../constants';
import { ILogOnModel, IAccountService } from '../account.api';
import { ACCOUNT_STATE } from '../account.constants';
import { IScope, IFormController } from 'angular';
import { IStateParamsService } from 'angular-ui-router';

class Controller {
    model: ILogOnModel;
    mode: string;
    displayContext: string;

    /*@ngInject*/
    constructor(private $scope: IScope,
                private accountService: IAccountService,
                public logOnLayout: ITypeLayout,
                $stateParams: IStateParamsService) {
    
        setPageTitle($scope, "Log on");
    
        this.mode = DETAILS_MODE.edit;
        this.displayContext = DISPLAY_CONTEXT.details;
        this.model = {
            ReturnUrl: $stateParams.ReturnUrl
        };
    }

    submit() {
        const logonForm: IFormController = this.$scope.logonForm; // form is not controller property due to wired exception
        if (!logonForm)
            throw "Logon form was not found";

        this.$scope.$broadcast(COMMON_EVENT.dirtifyPropertySuite);

        if (logonForm.$invalid)
            return;

        this.accountService.logonUser(this.model);
    };
}

register.state({
    name: ACCOUNT_STATE.logon,
    parent: ACCOUNT_STATE.noAccessLayout,
    url: "/logon?ReturnUrl",
    template: require('./logon.html'),
    resolve: {
        logOnLayout: /*@ngInject*/(accountService: IAccountService) => {
            return accountService.logOnLayout()
        }
    },
    suppressReloadMessage: true,
    controllerAs: "ctrl",
    controller: Controller,
}, { applicationStage: AC_APPLICATION_STAGE.formsAuthEnabled, useAsOtherwise: true });