'use strict';

import { register, setPageTitle } from '@systemorph/web';
import { AC_APPLICATION_STAGE } from '../../constants';
import { ACCOUNT_STATE } from '../account.constants';
import { IAccountService, IAuthenticationData } from '../account.api';
import { IScope } from 'angular';
import { IStateService, IStateParamsService } from 'angular-ui-router';

interface INotAuthenticatedStateParamsService extends IStateParamsService {
    ReturnUrl: string;
}

class Controller {
    message: string;

    /*@ngInject*/
    constructor($scope: IScope,
                authenticationData: IAuthenticationData,
                $state: IStateService,
                $stateParams: INotAuthenticatedStateParamsService) {
    
        setPageTitle($scope, "Not authenticated");
    
        this.message = 'You are not authorized to use this application. ';
    
        if (authenticationData.isForms) {
            const url = $state.href(authenticationData.logonStateName, { ReturnUrl: $stateParams.ReturnUrl || '/' });
    
            this.message += 'Please <a class="not-authorized-logon" href="' + url + '">logon</a>';
        } else {
            this.message += 'Please contact your system administrator.';
        }
    }
}

register.state({
    name: ACCOUNT_STATE.notAuthenticated,
    parent: ACCOUNT_STATE.noAccessLayout,
    url: '/notAuthenticated?ReturnUrl',
    template: require('./notAuthenticated.html'),
    controller: Controller,
    controllerAs: 'ctrl',
    resolve: {
        authenticationData: /*@ngInject*/(accountService: IAccountService) => {
            return accountService.getAuthenticationData();
        }
    },
    suppressReloadMessage: true,
}, { applicationStage: AC_APPLICATION_STAGE.notAuthenticated, useAsOtherwise: true });