'use strict';

import { IActivityLogNotification } from '@systemorph/web';
import { ui, IHttpService, blockUI } from 'angular';

//server-side model
interface IDeleteGroupValidationResult {
    notifications: IActivityLogNotification[]; //its a hack that activity log notifications are used here
    hasFatalError: boolean
}


export class DeleteGroupDialogController {
    title: string;
    message: string;
    okText: string;
    hideOk: boolean;
    notifications: IActivityLogNotification[]; 

    /*@ngInject*/
    constructor(private groupId: string,
                private $uibModalInstance: ui.bootstrap.IModalServiceInstance,
                private blockUI: blockUI.IBlockUIService,
                private $http: IHttpService) {
        this.title = "You are going to delete group. Continue?";
        this.message = "The group will no longer be an owner or a member of any group. Please make sure all members of this group have membership in some other groups.";
        this.okText = "Yes, delete it";
    }

    ok() {
        if (this.notifications) {
            this.performDeletion();
        } else {
            this.performValidation();
        }
    }

    private performValidation() {
        this.blockUI.start("Getting validations");
        this.$http.get<IDeleteGroupValidationResult>('/api/accessControlGroup/ValidateGroupDeletion', {params: {groupId: this.groupId}}).then(result => {
            this.blockUI.stop();
            
            if (this.isSuccessValidation(result.data)) {
                this.performDeletion();
                return;
            }

            this.applyValidationResult(result.data);
        })
    }

    private performDeletion() {
        this.blockUI.start("Performing deleteion");
        this.$http.post<IDeleteGroupValidationResult>(`/api/accessControlGroup/ConfirmGroupDeletion`, {}, {params: {groupId: this.groupId}}).then(result => {
            this.blockUI.stop();
            this.applyValidationResult(result.data);

            if (this.isSuccessValidation(result.data)) {
                this.$uibModalInstance.close();
                return;
            }
        })
    }

    private applyValidationResult(result: IDeleteGroupValidationResult) {
        this.notifications = result.notifications;
            this.hideOk = result.hasFatalError;
            this.okText = "Confirm";
            this.title = result.hasFatalError 
                            ?  "Deletion cannot be performed"
                            : "Please confim delete warnings";
    }

    private isSuccessValidation(result: IDeleteGroupValidationResult) {
        return !result.hasFatalError && !result.notifications.length;
    }

    cancel() {
        this.$uibModalInstance.dismiss();
    }
}