'use strict';

import { 
    register,
    IGridActionMenuItemScope,
    IEntityGridResult,
    MENU_PROPERTY
} from '@systemorph/web';

import { DeleteGroupDialogController } from './deleteGroupDialogController';
import { AC_GROUP_TYPE, AC_ACCESS_ASSIGNEMTN_TYPE } from "../accessControl"
import { IModalSettings, IModalService } from 'angular-ui-bootstrap';

interface IDeleteGroupGridActionMenuItemDirectiveScope extends IGridActionMenuItemScope {
    delete(): void;
    canDelete: boolean;
}

const groupTypePropertyName = "GroupType";
const assignmentTypePropertyName = "AssignmentType";

register.directive("deleteGroupGridActionMenuItem", () => {
    return {
        restrict: 'E',
        replace: true,
        template: `<button class="btn btn-transparent btn-inline btn-inline-big" ng-show="canDelete" ng-click="delete()" uib-tooltip="Delete Group" tooltip-append-to-body="true">
                      <span class="glyphicon glyphicon-trash"></span>
                   </button>`,
        controller: ($scope: IDeleteGroupGridActionMenuItemDirectiveScope, $uibModal: IModalService) => {
            const groupType: string = $scope.entity[groupTypePropertyName];
            const assignmentType: string = $scope.entity[assignmentTypePropertyName];
            $scope.canDelete = groupType !== AC_GROUP_TYPE.internal && assignmentType !== AC_ACCESS_ASSIGNEMTN_TYPE.directorySynchronized;

            $scope.delete = () => {
                if (!$scope.item.disabled && $scope.canDelete) {

                    const settings: IModalSettings = {
                        scope: $scope.$new(),
                        controller: DeleteGroupDialogController,
                        template: require('./deleteGroupDialog.html'),
                        size: 'lg',
                        controllerAs: "ctrl",
                        resolve: {
                            groupId: () => {
                                return $scope.entity.Id;
                            }
                        },
                        backdrop: true,
                        keyboard: true
                    };

                    return $uibModal.open(settings).result.then(() => {
                        const gridResult: IEntityGridResult = $scope.item.parameters[MENU_PROPERTY.gridResult];
                        gridResult.softUpdate();
                    });
                }

                return null;
            }
        }
    }
});