'use strict';

import { 
    register, DOM_EVENT, 
    IDataSourceService, ISystemorphEntity,
    IEntityService, IAutocompleteDimension, IPropertyOfEntityScope 
} from '@systemorph/web';

register.directive('editGroupConflictProperty', () => {
    return {
        restrict: 'E',
        scope: true,
        replace: true,
        controllerAs: 'ctrl',
        template: require('./editGroupConflictProperty.html'),
        controller: EditGroupConflictPropertyController
    };
});

type UiSelectContext = { search: string } | null;

// value of the property is string
// dropdown for the dimensions should be shown instead of textinput element. 
// SystemName of the dimension must be set to propertyValue when new item is selected at the combobox
class EditGroupConflictPropertyController {
    private readonly typeName = "Group";

    selectedValue: ISystemorphEntity;
    autocomplete: ISystemorphEntity[];

    constructor(private $scope: IPropertyOfEntityScope,
                private $element: ng.IAugmentedJQuery,
                private dataSourceService: IDataSourceService,
                private entityService: IEntityService) {

        const value: string = $scope.entity[$scope.propertyLayout.systemName];
        this.selectedValue = <any>{Id: null, SystemName: value, DisplayName: value};

        this.$element.on(DOM_EVENT.focus, 'input', () => {
            this.focus(null);
        });
    }

    onSelectionChanged(newVal: ISystemorphEntity, oldVal: ISystemorphEntity) {
        if (newVal && (oldVal == null || newVal.Id !== oldVal.Id)) {
            this.setValue(newVal);
        }
    }

    private setValue(value: ISystemorphEntity) {
        if (value) {
            this.$scope.entity[this.$scope.propertyLayout.systemName] = value.SystemName;
        }
    }

    private hasBeenTouched = false;

    // HACK start: autocmplete should be executed only on user focus
    focus(uiSelectContext: UiSelectContext) {
        if (uiSelectContext) {
            uiSelectContext.search = '';
        }

        if (!this.hasBeenTouched) {
            this.hasBeenTouched = true;
            this.refresh(null);
        }
    }
    // HACK end

    refresh(searchText: string) {
        // HACK start: whenever this coontrol first time loaded, we dont want to run autocomplete. Due to that we add hasBeenTouched flag in assumption
        if (!this.hasBeenTouched) {
            return;
        }
        // HACK end

        const dataSource = this.dataSourceService.getDataSourceFromScope(this.$scope);
        this.entityService.autocompleteDimension(searchText, null, this.$scope.propertyLayout.systemName, this.typeName, dataSource).then((items: IAutocompleteDimension[]) => {
            this.autocomplete = items.map(item => (<ISystemorphEntity>{ Id: item.id, SystemName: item.systemName, DisplayName: item.systemName }));
        });
    }
}