'use strict';

import { register, DISPLAY_CONTEXT, IMenuItemScope, IEntityCreationArgs, ICreateEntityService, ISystemorphEntity } from '@systemorph/web';
import { IAccessReviewService } from '../accessControl/accessControl.api';
import { AC_TYPE_NAME } from '../accessControl/accessControl.constants';

register.directive("scheduleAccessReviewMenuItem", () => {
    return {
        restrict: 'E',
        replace: true,
        scope: true,
        template: `<a ng-click="ctrl.schedule()" ng-bind-html="ctrl.label"></a>`,
        controllerAs: "ctrl",
        controller: MenuController
    }
});

class MenuController {
    label: string;

    constructor(private $scope: IMenuItemScope,
                private accessReviewService: IAccessReviewService,
                private createEntityService: ICreateEntityService) {
        this.label = this.$scope.item.label || "Schedule access review";
    }

    schedule() {
        const args: IEntityCreationArgs = {
            additionalPostChanges: null,
            additionalPreChanges: null,
            dialogTitle: "Schedule access review",
            displayContext: DISPLAY_CONTEXT.details,
            typeName: AC_TYPE_NAME.accessReview,
            parentId: null
        }
        this.createEntityService.create(args).then((entity: ISystemorphEntity) => {
            this.accessReviewService.notifyReviewCreated(entity.Id);
        })
    }
}